@@ -122,10 +122,21 @@ class mcDETECT:
                     continue
                 temp_in_nucleus = np.sum(target["overlaps_nucleus"].values[mask])
                 temp_size = coords.shape[0]
-                # coords_unique = np.unique(coords, axis=0)
+                
                 temp = pd.DataFrame(coords, columns=["global_x", "global_y", "global_z"])
                 temp = temp.drop_duplicates()
-                center, r2 = miniball.get_bounding_ball(np.array(temp), epsilon=1e-8)
+                coords_unique = temp.to_numpy()
+                if coords_unique.shape[0] < self.low_bound:
+                    print(f"Skipping small cluster for gene {j}, cluster {k} (n = {coords_unique.shape[0]})")
+                    continue
+                try:
+                    center, r2 = miniball.get_bounding_ball(coords_unique, epsilon=1e-8)
+                except np.linalg.LinAlgError:
+                    print(f"Warning: singular matrix for gene {j}, cluster {k} —- using fallback sphere.")
+                    center = coords_unique.mean(axis=0)
+                    dists = np.linalg.norm(coords_unique - center, axis=1)
+                    r2 = (dists.max() ** 2)
+                
                 if self.type != "Xenium":
                     closest_z = closest(z_grid, center[2])
                 else:
